;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x48730f49baa9a5f9b2f909a4a64228e5 . "./code.cg2js/cg-status-bar.fasl")
(#x65dfbdb46b2da865c17548ea2c0a16b3 . "./code.cg2js/cg-hotspot.fasl")
(#x3a5fb2bc545b765519c275bd7ea5feff . "./code.cg2js/cg-toolbar.fasl")
(#x3e17373bd10282e3b38c21c9ec510142 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x0a2f8a06e09a7bfb84484f35f1dbd022 . "./code.cg2js/cg-tooltip.fasl")
(#x2dd5970d2423da4d89df0882fcf525cb . "./code.cg2js/cg-calendar.fasl")
(#x5fe294aa76ef4b4cabd206acf8175e57 . "./code.cg2js/cg-item-list.fasl")
(#x922dfbe424db2f734b98f49543a8580e . "./code.cg2js/cg-bitmap-stream.fasl")
(#xc161820d44b9aab98cb53ae1c58c7295 . "./code.cg2js/cg-trackbar.fasl")
(#x1a3900f63baedda7bacff17f0065553b . "./code.cg2js/cg-class-grid.fasl")
(#xfeb1dd2e678918be2633de5368bcf2e0 . "./code.cg2js/cg-dialog-item.fasl")
(#x9e78518dabdbb6d25ea4a5381076084f . "./code.cg2js/cg-comtab.fasl")
(#x96a6501f802cfeb2b0266dc59098af8a . "./code.cg2js/cg-lisp-text.fasl")
(#x12f228fb9dce92f4faf29462d53789fc . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x9a731071d7c61867f91d5cb9ee4f386a . "./code.cg2js/cg-button.fasl")
(#x3b543969bb7804a69300c424cac57743 . "./code.cg2js/cg-printing.fasl")
(#x11397bc6d14ecccbc2c0df6afd0f5b6b . "./code.cg2js/cg-pixmap-printing.fasl")
(#xcae6e16c9b3e218ee117aa37ada03f62 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x31810689be9ae1d65840d7087cf6fad8 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x552b77e9a0ece4385915d330f3141d4d . "./code.cg2js/cg-selected-object.fasl")
(#x411f15181e642166adff0171ae2741f1 . "./code.cg2js/cg-text-or-combo.fasl")
(#xc365fd8b4e24fff88b2ecbb5a870deaf . "./code.cg2js/cg-graphics-context.fasl")
(#xe63f45518cc2e28a01384aeb01a9b261 . "./code.cg2js/cg-edit-in-place.fasl")
(#x2753c3d2712a66a9106fe4430d247124 . "./code.cg2js/cg-string-dialog.fasl")
(#x9907fdea23224acd733019839d0b335e . "./code.cg2js/cg-list-view.fasl")
(#x2c7f426dc79eb576f0fc8e9041800674 . "./code.cg2js/cg-intersect.fasl")
(#x26e8d3a64001f85fb5433edf4b5be18d . "./code.cg2js/cg-text-widget.fasl")
(#x40f30f3df8342cc5ca0f4371ee92c15b . "./code.cg2js/cg-directory-dialog.fasl")
(#x9de3a635668c2483d2511f4bfa4c25c0 . "./code.cg2js/cg-file-dialog.fasl")
(#x654e79b8e448e74e2a926dc06a131aab . "./code.cg2js/cg-dde-utils.fasl")
(#x18314aee84ca9e8238722ba6502d3214 . "./code.cg2js/cg-nodes.fasl")
(#xd7ba13fc8e327d7a42181fea0bdd378d . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x60465cedc2608a42d7d22cc79e722b04 . "./code.cg2js/cg-up-down-control.fasl")
(#xcd95bf71670f8b4f5f23db66b1053619 . "./code.cg2js/cg-get-position.fasl")
(#xadb0f909cd2f77161abe7d970b3f2f4d . "./code.cg2js/cg-toggling-widget.fasl")
(#x183eb1d34ee3deb1b2c384a0df276a4e . "./code.cg2js/cg-acache.fasl")
(#x16dc7064ea1188418cc260c2c59f1230 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x64aa009a2868651d0232d6d965661547 . "./code.cg2js/cg-base.fasl")
(#xeaa03660ed8b2ff821d825f7cbd713d9 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x24b88c4d8413e30a1cd77f9038b7a944 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xfae54ff01f3ad1d01623716a34db2465 . "./code.cg2js/cg-property.fasl")
(#xe2934e51eb483c5f346db08f631d0582 . "./code.cg2js/cg-editable-text.fasl")
(#x4764680b3af63e931c37d7988565c8f8 . "./code.cg2js/cg-lamp.fasl")
(#xe38920b0b3e59178ee75bbb90082fa53 . "./code.cg2js/cg-progress-indicator.fasl")
(#x20d16b003abb97712e3da83dbf7481b1 . "./code.cg2js/cg-chart-widget.fasl")
(#x721d3f24139888e44a554bfe2ec2b2ef . "./code.cg2js/cg-common-control.fasl")
(#x2def188d0c2bf110f469173d6cdf9e22 . "./code.cg2js/cg-font-dialog.fasl")
(#xbc353a30d70f8b965ec6c0f3c69a51cc . "./code.cg2js/cg-gesture-emulation.fasl")
(#x070631fd9806acee296aaa994c441d67 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x6fcc46774d34d22432b147c02aeec3ff . "./code.cg2js/cg-os-window.fasl")
(#x440c80672d7b9c0a04646b0eebbff2c3 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xd7b099cb6e99842aa1984dd7bb5ac82d . "./code.cg2js/cg-pixmap.fasl")
(#x7f0e9d9ce57eabfa57b2dbc3e925aba0 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x31566a6f33a59527420589bc902234f5 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x6874c5b58228eb48383371badc5c1200 . "./code.cg2js/cg-scaling-stream.fasl")
(#xb00c73de69bd8dc0f6276a8fb504f18b . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x7c813f5e72fe814cbc6c6c37cce18482 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xfa42905b605b83deb2741c798ee10b4f . "./code.cg2js/cg-timer.fasl")
(#x07197335dc7738fad27308a17538fb7b . "./code.cg2js/cg-scroll-bar.fasl")
(#xed7d31e80a1c313e64f7e8f9e23ea83a . "./code.cg2js/cg-caret.fasl")
(#x303cb7915e876a8bc4119bfba8aa625e . "./code.cg2js/cg-template-string.fasl")
(#x21f1a74c7af305ed14bf2edd942ffcd2 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x0ea9d272721896dad508a1b40034f0d7 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x51b9e3513d796bcc89fab62ac471b584 . "./code.cg2js/cg-rich-edit.fasl")
(#x16e98600ff88b6ea5df56fe091a831df . "./code.cg2js/cg-combo-box.fasl")
(#xbc9b46f42c738af4a415bf2e85bb58d2 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x0ec88982485241dfffd172d9ba80565e . "./code.cg2js/cg-multi-picture-button.fasl")
(#xb9927985e8636a90893865700f53ff13 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xaaffbdfe970a2126de2cbf0838533fef . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x00bc29682ac159701ee2d7b486b03f9b . "./code.cg2js/cg-outline.fasl")
(#x5b94b48a1fb962db8e030d37e08071e5 . "./code.cg2js/cg-class-support.fasl")
(#xb8434d5faa4607bc86bebf5451a9eaf8 . "./code.cg2js/cg-lettered-menu.fasl")
(#x2536c2dc687cbe01f61283c629419852 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x7bc2cf32a70b5cb2497b1d9a6196ef13 . "./code.cg2js/cg-fill-texture.fasl")
(#xcb420a729eb0353fd830209f23103422 . "./code.cg2js/cg-lisp-widget.fasl")
(#x14f8508ad2d22a7133da8e2677a592d4 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x76aa58ab87f40545498d9b7c4194f3f2 . "./code.cg2js/cg-clipboard.fasl")
(#xc78ec1483edfad19ee5c7930d1e6eae8 . "./code.cg2js/cg-wrap-string.fasl")
(#x8d2cc369a384a735330b828a13b363f1 . "./code.cg2js/cg-os-widget.fasl")
(#x6df0cfdb1de816652d28595d92bbedc2 . "./code.cg2js/cg-check-box.fasl")
(#x80e4abbd5c3e82145e9e3d2a1eb393af . "./code.cg2js/cg-choice-list.fasl")
(#x490dda2c0a5cb331a7345ab24730e585 . "./code.cg2js/cg-layout.fasl")
(#xd32ecaed82d9395d855f094c365a4d1c . "./code.cg2js/cg-group-box.fasl")
(#x26708d6edfbd7e8d2c8482d9ddf46df6 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x94e58ae88df056b7062fcc93e394cc01 . "./code.cg2js/cg-radio-button.fasl")
(#xf1f65a6312773fe5a4f381e752254258 . "./code.cg2js/cg-tab-control.fasl")
(#xb371a7fa87ce6f59af24706bf531856f . "./code.cg2js/cg.fasl")
(#x8f3a27242a15d2e07d9fcd9fde76201a . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x8f98d5dc80bd4217b7183f5fe50f5b32 . "./code.cg2js/cg-dropping-outline.fasl")
(#x6d9d35b55a5ce329d851267fae2fd38f . "./code.cg2js/cg-get-pixmap.fasl")
(#xc8ec661f307069d58db7355e43833ab0 . "./code.cg2js/cg-grid-widget.fasl")
(#x1267faf7b7f4c80e81b0e38b6208339f . "./code.cg2js/cg-utility-dialog.fasl")
(#xbb3345306edaf9c0f1449ee5ea80cf1b . "./code.cg2js/cg-static-text.fasl")
(#x0dee807e2658e10ad6502221dd181cdc . "./code.cg2js/cg-class-slot-grid.fasl")
(#xab6399ecc01c22e607e7f3f257d61aaa . "./code.cg2js/cg-object-editor-layout.fasl")
(#x66933c8145d0a9ee9781842a5b13892e . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xa2001e59479af43addce2f37a449fe97 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x67c931e1dd913bd4a2f3cafd18132fa6 . "./code.cg2js/cg-plot-widget.fasl")
(#xe79cb71f448ab2832e2fcefe9069da2c . "./code.cg2js/cg-color-dialog.fasl")
(#x7a7c05be7bcc7f9ad45769c2613289e1 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x7511ed5e79482606b5c545695ebb1227 . "./code.cg2js/cg-icon.fasl")
(#xce34ed124ecdf2976760fbdadcf08313 . "./code.cg2js/cg-dev.fasl")
(#x8f193aaf6bd09fa874a4e66e442267d8 . "./code.cg2js/cg-choose-printer.fasl")
(#xf8c87011403f0022972d28a18fcae01e . "./code.cg2js/cg-intersect-posbox.fasl")
(#x09464e4e38b2cce2f4c7bbecdbdbf452 . "./code.cg2js/cg-menu.fasl")
(#xc40c33173cee848ac3c329d4ffdef3ff . "./code.cg2js/cg-yes-no-list.fasl")
(#x12e4a04040cbbc7223f3b40b9a08162a . "./code.cg2js/cg-palette.fasl")
(#x3fb02dd8924892bcb47f86a0d0f05f69 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x6a95e069e39b390e5d1d740a44689c74 . "./code.cg2js/cg-message-dialog.fasl")
(#xe3567b39b98632199fbc39ee62d6b0ba . "./code.cg2js/cg-shortcut-menu.fasl")
(#xfb00f397d1cdaff15475b6a076c323d4 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x21dccd3823e78db688c9e8d29bfbd07f . "./code.cg2js/cg-split-bar.fasl")
(#xe71d6ce286342b1d59289c84c88f405f . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x7dd7f306246e360d4cfb12515a9615f2 . "./code.cg2js/cg-web-browser.fasl")
(#x1becc91a316950f1de035571b7ae121b . "./code.cg2js/cg-project-window.fasl")
(#x4db5998a2a804747042b7bb12d30530d . "./code.cg2js/cg-picture-widget.fasl")
(#x57443b07182114b652d79155f7464b08 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x260faf29d713b0e2065b38fec4473ce3 . "./code.cg2js/cg-object-editor.fasl")
(#x6bf7644299cea8402cac22e6672058b0 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x22507b35f720d2d611595073d21057a8 . "./code.cg2js/cg-paren-matching.fasl")
(#xdc8d500ffb1f48f96b0cc3c05f47167c . "./code.cg2js/cg-sample-file-menu.fasl")
(#xe41909419c4377f6e8664951e647cd92 . "./code.cg2js/cg-drawable.fasl")
(#xa7b5c0dc307a37393da3a9ed0eacaca2 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xe2c2674be951add5d564d5852bb9709c . "./code.cg2js/cg-html-widget.fasl")
(#x9c96776a165402d75702146c0b5cfe02 . "./code.cg2js/cg-curve.fasl")
(#x4eef9d6beea9fc79e5c70625bb4ad56b . "./code.cg2js/cg-yes-no-string.fasl")
(#xc90d6a922e8c62467f1564651f2c5f05 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x34a464ea3526ab7f4d37bfe8766af31f . "./code.cg2js/cg-html-dialog.fasl")
(#xf9528a8e43ca0cfd9f81fc77971dfb3a . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x726fce90e34b7942d87dd027e6862376 . "./code.cg2js/cg-tray.fasl")
(#xbdff2e7768d7280dbb819c0a90dee27b . "./code.idejs/ide-apropos.fasl")
(#xa9e4a6d284092243be785253ba4c04a3 . "./code.idejs/ide-window-list.fasl")
(#xfd01aa613ec96cc0bb4d26c39628cb5a . "./code.idejs/ide-class-browser.fasl")
(#xba95d0c52d519e90c2ada2ef178d0b6b . "./code.idejs/ide-base.fasl")
(#x522db101bec8ca7dd38b8de75b94816e . "./code.idejs/ide-process-browser.fasl")
(#x2a36facc5b2b1571fcd73244e7e9b5f8 . "./code.idejs/ide-acache.fasl")
(#x9fc2223d9dbe04228d1c1aabf17504db . "./code.idejs/ide-navigator.fasl")
(#x31780b4028ad3d6fd6b0662830013af5 . "./code.idejs/ide-trace.fasl")
(#xd8fb6339d510ec11795ee7e2e9b8b21f . "./code.idejs/ide-project.fasl")
(#x97d814ae6d9ff145b44f1872c93bd327 . "./code.idejs/ide-patch-dialog.fasl")
(#xb591127973a224ee34aceacc9ba4525f . "./code.idejs/ide-grapher.fasl")
(#xf3afdc9e044c0fd1636da6023090c71d . "./code.idejs/ide-dev.fasl")
(#x649736927c2871042c57f9032b8ccf3d . "./code.idejs/ide-keyshort.fasl")
(#x61f0c2bab6e1288bf02905069ac6b3ff . "./code.idejs/ide-editor.fasl")
(#xdecdfc93e7f1211934601e67ed5208b6 . "./code.idejs/ide-inspector.fasl")
(#x2d94d686f5613f6673acd3badfeee244 . "./code.idejs/ide.fasl")
(#x7f7d91efb40c91944f5d289f9cb87fce . "./code.idejs/ide-clipboard.fasl")
(#xcfdef1f09de7a54c28bfa5cf222505aa . "./code.idejs/ide-find-definitions.fasl")
(#x68bb38cc9b2de7ec2bcf8049d92bf1b6 . "./code.idejs/ide-form.fasl")
(#x906024b5729920e76c2930ae74a844a8 . "./code.idejs/ide-intro.fasl")
(#x9ec62edb5c29230317b28f166a754ab2 . "./code.idejs/ide-find-in-files.fasl")
(#x4264c7015ee65994270e903747f66893 . "./code.idejs/ide-package-list.fasl")
(#x7472895700aeb968c8646037f1bd0957 . "./code.idejs/ide-class-grapher.fasl")
(#x49a17b9a618997e974ff2e9af0f2eb06 . "./cgjs.html")
(#x44f169492307df5920f609d11cc1a34c . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
